// thundervideo.h

#pragma once

using namespace System;
using namespace System::Drawing;

namespace thundervideo {

    class ThunderVideoNativeCtx;

    public enum class ThunderVideoError { OK, FAILED_TIMEOUT, FAILED_OTHER };

    // SINGLE THREADED ACCESS ONLY
	public ref class ThunderVideo
	{
    public:
        ThunderVideo();
        ~ThunderVideo();
        !ThunderVideo();

        // Return values:
        // OK: All good, ready to go
        // FAILED_TIMEOUT: timeoutMs went by without any new data or getting enough data
        //                 setup stream
        // FAILED_OTHER: Already open or some other error occurred
        ThunderVideoError open(int port, int timeoutMs);

        // Safe to call in any state; no-op if not open
        void close();

        // Try to read and decode the next frame, waiting up to the given # of milliseconds
        // Returns Bitmap of next frame
        // Return values:
        // OK: All good, image data is set
        // FAILED_TIMEOUT: timeoutMs went by without any new data or new frame being decoded
        // FAILED_OTHER: Any other error
        ThunderVideoError getNextFrame(Bitmap ^%image, int timeoutMs);
    private:
        ThunderVideoNativeCtx *nativeCtx;
        static bool staticInit();
	};
}
